@@ -388,6 +388,12 @@ static void _compiler_callback(
 }
 
 int compile_rule(char* string, YR_RULES** rules)
+{
+  bool strict_escape_flag = false;
+  return compile_rule_ex(string, rules, strict_escape_flag);
+}
+
+int compile_rule_ex(char* string, YR_RULES** rules, bool strict_escape_flag)
 {
   YR_COMPILER* compiler = NULL;
   int result = ERROR_SUCCESS;
@@ -401,6 +407,8 @@ int compile_rule(char* string, YR_RULES** rules)
     goto _exit;
   }
 
+  compiler->strict_escape = strict_escape_flag;
+
   yr_compiler_set_callback(compiler, _compiler_callback, &warnings);
 
   // Define some variables that will be used in test cases.
@@ -574,8 +582,9 @@ static int capture_matches(
       yr_string_matches_foreach(context, string, match)
       {
         if (strlen(f->expected) == match->data_length &&
-            strncmp(f->expected, (char*) (match->data), match->data_length) ==
-                0)
+            (match->data_length == 0 ||
+             strncmp(f->expected, (char*) (match->data), match->data_length) ==
+                 0))
         {
           f->found++;
         }
@@ -711,7 +720,7 @@ void assert_re_atoms(char* re, int expected_atom_count, atom* expected_atoms)
 
   int exit_code;
 
-  yr_re_parse(re, &re_ast, &re_error);
+  yr_re_parse(re, &re_ast, &re_error, RE_PARSER_FLAG_NONE);
   exit_code = _assert_atoms(re_ast, expected_atom_count, expected_atoms);
 
   if (re_ast != NULL)
